@@ -11,7 +11,7 @@
 #endif
 #include <sys/time.h>
 
-using namespace boost;
+#include "namespaces.hh"
 using namespace std;
 
 class KqueueFDMultiplexer : public FDMultiplexer
@@ -93,12 +93,12 @@ int KqueueFDMultiplexer::run(struct timeval* now)
   ts.tv_nsec=500000000U;
 
   int ret=kevent(d_kqueuefd, 0, 0, d_kevents.get(), s_maxevents, &ts);
-  gettimeofday(now,0);
+  gettimeofday(now,0); // MANDATORY!
   
   if(ret < 0 && errno!=EINTR)
     throw FDMultiplexerException("kqueue returned error: "+stringerror());
 
-  if(ret==0) // nothing
+  if(ret < 0) // nothing - thanks AB!
     return 0;
 
   d_inrun=true;
@@ -107,6 +107,7 @@ int KqueueFDMultiplexer::run(struct timeval* now)
     d_iter=d_readCallbacks.find(d_kevents[n].ident);
     if(d_iter != d_readCallbacks.end()) {
       d_iter->second.d_callback(d_iter->first, d_iter->second.d_parameter);
+      continue; // so we don't find ourselves as writable again
     }
 
     d_iter=d_writeCallbacks.find(d_kevents[n].ident);
